"""
    GeoBound
(PT-BR)
    Identificação de coordenadas e geolocalização de polígonos e linhas.
(EN-US)
    Identification of coordinates and geolocation of polygons and lines.

Copyright (C) 2024 Thiago Daniel Pessoa

This program is free software: you can redistribute it and/or modify it _
under the terms of the GNU General Public License as published by the   _
Free Software Foundation, either version 3 of the License, or (at your  _
option) any later version.

This program is distributed in the hope that it will be useful, but     _
WITHOUT ANY WARRANTY; without even the implied warranty of              _
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        _
General Public License for more details.

You should have received a copy of the GNU General Public License along _
with this program. If not, see <https://www.gnu.org/licenses/>.
"""


def point_in_poly(point: tuple, polygon: list) -> bool:
    """
    Iterates through the `polygon`'s items, counting if the ray of    _
    intersection calculated for item and the point is greater than    _
    the point's first value. When the count ends in an odd number,    _
    the point is inside the polygon.
    :param point: <tuple(2)> of latitude and longitude
    :param polygon: <list(n)<tuple(2)>> of latitudes and longitudes
    :return: Whether the point is inside or outside the polygon.
    """
    point_to_end_right_segment = (point, (point[0], 180))
    is_inside = False
    for i in range(len(polygon)):
        current_segment = (polygon[i], polygon[(i + 1) % len(polygon)])
        # The Mod is  used to ensure that the zero index value is
        # called when the index for the last item is reached.
        if verify_intersection(point_to_end_right_segment, current_segment):
            is_inside = not is_inside

    return is_inside


def verify_collinearity(point_a: tuple, segment_bc: tuple) -> bool:
    """
    Tells whether a point lies on a line segment.

    :param point_a: The coordinate point (latitude, longitude) to be
    checked for collinearity with the segment.
    :param segment_bc: Two endpoints of the line segment (latitudes,
    longitudes) to which the point may be collinear.
    :return: True if the point is collinear with the segment, False
    otherwise.
    """
    segment_ab = (point_a, segment_bc[0])
    slope_bc = calculate_slope(segment_bc)
    slope_ab = calculate_slope(segment_ab)
    return slope_ab == slope_bc


def round_point(point: tuple) -> tuple:
    """
    Rounds the point's longitude and latitude to the 6th decimal place.
    :param point: <tuple(2)> of longitude and latitude.
    :return: The rounded coordinates.
    """
    return (
        round(point[0], 6),
        round(point[1], 6)
    )


def calculate_slope(segment: tuple) -> float:
    """
    Divides the vertical change (rise) by the horizontal change (run)
    of the segment.
    :param segment: <tuple(2)<tuple(2)>> of two coordinates points.
    :return: the value of the slope between the points.
    """
    x_a, y_a = round_point(segment[0])
    x_b, y_b = round_point(segment[1])
    if x_a == x_b:
        slope = 0

    else:
        slope = (y_b - y_a) / (x_b - x_a)

    return round(slope, 2)


def verify_intersection(segment_a: tuple, segment_b: tuple) -> bool:
    """
    Tells whether two lines crosses.

    :param segment_a: First line's coordinates points for start and end.
    :param segment_b: Second line's coordinates points for start and end.
    :return: True if the lines crosses and False if not.
    """
    # Breaking the segments into its coordinate points.
    lineA_pointA, lineA_pointB = segment_a
    lineB_pointA, lineB_pointB = segment_b

    # Calculating the vectors for the lines
    vector_A = (
        lineA_pointB[0] - lineA_pointA[0],
        lineA_pointB[1] - lineA_pointA[1]
    )
    vector_B = (
        lineB_pointB[0] - lineB_pointA[0],
        lineB_pointB[1] - lineB_pointA[1]
    )

    determinant = vector_A[0] * vector_B[1] - vector_A[1] * vector_B[0]
    if determinant == 0:
        return False  # The segments are parallel to each other

    # Calculating the parameters s and t
    s = (
        (
            (lineB_pointA[0] - lineA_pointA[0]) * vector_B[1] -
            (lineB_pointA[1] - lineA_pointA[1]) * vector_B[0]
        ) / determinant
    )
    t = (
        (
            (lineB_pointA[0] - lineA_pointA[0]) * vector_A[1] -
            (lineB_pointA[1] - lineA_pointA[1]) * vector_A[0]
        ) / determinant
    )

    return 0 <= s <= 1 and 0 <= t <= 1



"""
    GeoBound
(PT-BR)
    Identificação de coordenadas e geolocalização de polígonos e linhas.
(EN-US)
    Identification of coordinates and geolocation of polygons and lines.

Copyright (C) 2024 Thiago Daniel Pessoa

This program is free software: you can redistribute it and/or modify it _
under the terms of the GNU General Public License as published by the   _
Free Software Foundation, either version 3 of the License, or (at your  _
option) any later version.

This program is distributed in the hope that it will be useful, but     _
WITHOUT ANY WARRANTY; without even the implied warranty of              _
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        _
General Public License for more details.

You should have received a copy of the GNU General Public License along _
with this program. If not, see <https://www.gnu.org/licenses/>.
"""

from utils import treat_data, geometry, io


class Area:
    """
    Represents a collection of coordinates that forms a geospatial
    polygon.
    """
    def __init__(self, title='', polygon=None, file_name=''):
        if file_name != '':
            title, polygon = self._get_from_file(file_name)

        self.title = title
        self.polygon = treat_data.digest_coordinates(self.title, polygon)
        assert self.polygon[0] == self.polygon[-1], (
            f'The polygon of {self.title} is open.'
        )

    @property
    def borders(self) -> list:
        """
        Every pair of coordinates in the polygon that are sequential.
        """
        borders = []
        for i in range(len(self.polygon) - 1):
            border = (self.polygon[i], self.polygon[i + 1])
            borders.append(border)
        return borders

    def verify_containment(self, coordinate: tuple) -> bool:
        """
        Asserts that the coordinate is valid, verify the coordinate's _
        appearance in its polygon, run the calculations necessary to  _
        verify if the point is inside the polygon.
        :param coordinate: tuple(latitude, longitude)
        :return: Boolean value for the point being part of the polygon_
        or its area
        """
        treat_data.check_coordinate(coordinate)
        return geometry.point_in_poly(coordinate, self.polygon)

    def verify_borderline(self, coordinate: tuple) -> bool:
        """
        Iterates through the borders of the object, verifying if the  _
        coordinate point is collinear to the segments.
        :param coordinate: A <tuple(2)> of latitude and longitude.
        :return: Whether the point is on the border of Self or not.
        """
        for border in self.borders:
            if geometry.verify_collinearity(coordinate, border):
                return True

        return False

    @staticmethod
    def _get_from_file(path: str) -> tuple:
        """
        Gets the data needed from a `.geojson` or `.json` file.
        :param path: The path to the file
        """
        feature = io.get_single_area(path)
        return (
            feature[0]['properties']['name'],
            feature[0]['geometry']['coordinates'][0]
        )



"""
    GeoBound
(PT-BR)
    Identificação de coordenadas e geolocalização de polígonos e linhas.
(EN-US)
    Identification of coordinates and geolocation of polygons and lines.

Copyright (C) 2024 Thiago Daniel Pessoa

This program is free software: you can redistribute it and/or modify it _
under the terms of the GNU General Public License as published by the   _
Free Software Foundation, either version 3 of the License, or (at your  _
option) any later version.

This program is distributed in the hope that it will be useful, but     _
WITHOUT ANY WARRANTY; without even the implied warranty of              _
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        _
General Public License for more details.

You should have received a copy of the GNU General Public License along _
with this program. If not, see <https://www.gnu.org/licenses/>.
"""

import json

VALID_EXTENSIONS = ['geojson', 'json']
MESSAGES = {
    0: "The file's extension is unsupported: [`{}`].",
    1: 'The file `{}` must contain exactly one feature and it must be a '
       '`Polygon`.',
    2: 'The file `{}` must contain exactly one feature and it must be a '
       '`LineString`.',
    3: 'The file `{}` must contain at least one structure'
}


class InvalidFileExtension(Exception):
    """
    Handles the Exception call if an invalid extension is passed.
    """
    def __init__(self, file_name: str):
        message = MESSAGES[0].format(file_name)
        super().__init__(message)


def extract_data_from_file(file_path: str) -> str:
    """
    Reads the lines in the file and returns it as one single string.
    :param file_path: Path and name to the file
    :return: Text of the file
    """
    check_extension(file_path)
    file_data = ''
    try:
        with open(file_path, 'r', encoding='utf-8') as file:
            for line in file.readlines():
                file_data += line
    except FileNotFoundError:
        pass

    return file_data


def get_single_area(file_path: str) -> None | dict:
    """
    Collects the data from a file containing a single area.
    :param file_path: Path and name to the file
    :return: The json in the file data
    """
    file_data = extract_data_from_file(file_path)
    features = json.loads(file_data)['features']
    assert len(features) == 1 and file_data.count('Polygon') == 1, (
        MESSAGES[1].format(file_path)
    )
    return features


def get_multiple_areas(file_path: str) -> None | dict:
    """
    Collects the data from a file containing more than one area.
    :param file_path: Path and name to the file.
    :return: A dict containing the name of each structure as key and
    its perimeter as value.
    """
    file_data = extract_data_from_file(file_path)
    features = json.loads(file_data)['features']
    assert len(features) >= 1, (
        MESSAGES[3].format(file_path)
    )
    items_found = {}
    for feature in features:
        geometry = feature['geometry']
        if geometry['type'] == 'Polygon':
            name = feature['properties']['Name']
            coordinates = geometry['coordinates'][0]
            items_found[name] = coordinates

    return items_found


def get_single_line(file_path: str) -> None | dict:
    """
    Collects the data from a file containing a single line.
    :param file_path: Path and name to the file.
    :return: The json in the file data.
    """
    file_data = extract_data_from_file(file_path)
    features = json.loads(file_data)['features']
    assert len(features) == 1 and file_data.count('LineString') == 1, (
        MESSAGES[2].format(file_path)
    )
    return features


def get_multiple_lines(file_path: str) -> None | dict:
    """
    Collects the data from a file containing more than one line.
    :param file_path: Path and name to the file.
    :return: A dict containing the name of each structure as key and
    its shape as value.
    """
    file_data = extract_data_from_file(file_path)
    features = json.loads(file_data)['features']
    assert len(features) >= 1, (
        MESSAGES[3].format(file_path)
    )
    items_found = {}
    for feature in features:
        geometry = feature['geometry']
        if geometry['type'] == 'LineString':
            name = feature['properties']['Name']
            coordinates = geometry['coordinates']
            items_found[name] = coordinates

    return items_found


def check_extension(file_path: str) -> None:
    """
    Checks the path for the valid extensions.
    :param file_path: Path and name to the file
    """
    ext = file_path.split('.')[-1]
    if ext not in VALID_EXTENSIONS:
        raise InvalidFileExtension(file_path)



"""
    GeoBound
(PT-BR)
    Identificação de coordenadas e geolocalização de polígonos e linhas.
(EN-US)
    Identification of coordinates and geolocation of polygons and lines.

Copyright (C) 2024 Thiago Daniel Pessoa

This program is free software: you can redistribute it and/or modify it _
under the terms of the GNU General Public License as published by the   _
Free Software Foundation, either version 3 of the License, or (at your  _
option) any later version.

This program is distributed in the hope that it will be useful, but     _
WITHOUT ANY WARRANTY; without even the implied warranty of              _
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        _
General Public License for more details.

You should have received a copy of the GNU General Public License along _
with this program. If not, see <https://www.gnu.org/licenses/>.
"""

from utils import treat_data, io


class Line:
    """
    Represents a collection of coordinates that forms a geospatial line.
    """
    def __init__(self, title='', shape=None, file_name=''):
        if file_name != '':
            title, shape = self._get_from_file(file_name)

        self.title: str = title
        self.shape: list = treat_data.digest_coordinates(self.title, shape)
        self._check_line()

    @property
    def segments(self) -> list:
        """
        An organized form of all segments that forms the object.
        :returns: Tuples of two sequential coordinates from the object
        that forms a segment.
        """
        segments = []
        for i in range(len(self.shape) - 1):
            segment = (self.shape[i], self.shape[i + 1])
            segments.append(segment)
        return segments

    def _check_line(self) -> None:
        """
        Verify for duplicates of the coordinates in the shape.
        Raises AssertionError if any is found.
        """
        for point in self.shape:
            assert self.shape.count(point) == 1, (
                'The line has duplicated points.'
            )

    @staticmethod
    def _get_from_file(path: str) -> tuple:
        """
        Extracts the data from the json/geojson file passed and       _
        returns the shape and name of the first line found.
        :param path: The path of the file that has the data
        :return:  A <tuple(2)> with the name and coordinates set of   _
        the line.
        """
        features = io.get_single_line(path)
        return (
            features[0]['properties']['Name'],
            features[0]['geometry']['coordinates']
        )



"""
    GeoBound
(PT-BR)
    Identificação de coordenadas e geolocalização de polígonos e linhas.
(EN-US)
    Identification of coordinates and geolocation of polygons and lines.

Copyright (C) 2024 Thiago Daniel Pessoa

This program is free software: you can redistribute it and/or modify it _
under the terms of the GNU General Public License as published by the   _
Free Software Foundation, either version 3 of the License, or (at your  _
option) any later version.

This program is distributed in the hope that it will be useful, but     _
WITHOUT ANY WARRANTY; without even the implied warranty of              _
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        _
General Public License for more details.

You should have received a copy of the GNU General Public License along _
with this program. If not, see <https://www.gnu.org/licenses/>.
"""

from Area import *
from Line import *

MESSAGES = {
    0: 'At least one file must be passed.',
    1: 'Dispersed Charted Field.',
    2: 'Outer area of {}.',
    3: 'Out of charted field.',
    4: 'The point matches {} statement{}: ',
    5: 'Located inside {}',
    6: 'Located on the border of {}',
    7: 'Located on the line {}',
}


class Map:
    """
    Represents a collection of Lines and Areas that forms a
    geospatial map.
    """
    def __init__(
            self, map_name: str, file_map='none.geojson',
            file_areas='none.geojson', file_lines='none.geojson'
    ):
        assert (
                file_areas != 'none.geojson' or
                file_lines != 'none.geojson' or
                file_map != 'none.geojson'
        ), MESSAGES[0]
        io.check_extension(file_map)
        io.check_extension(file_areas)
        io.check_extension(file_lines)
        self.map_name = map_name

        if file_areas == 'none.geojson' == file_lines:
            self._gather_map(file_map)
        elif file_areas != 'none.geojson':
            self.areas = self._gather_areas(file_areas)
        if file_lines != 'none.geojson':
            self.lines = self._gather_lines(file_lines)

        self.map_area = self._join_areas()
        self._sort_areas()

    def locate_point(self, coordinate: tuple, simplify=False) -> str:
        """
        Checks for appearances of the point inside and in the borders _
        of every area in areas and in the line segments of every line _
        in lines.
        :param coordinate: tuple(latitude, longitude).
        :param simplify: boolean to alter the return value from a     _
        verbose (default) to a simple, easier to handle, result.
        :return: A message saying where the point was found or is _
        out of the charted field.
        """
        found_data = []
        for area in self.areas:
            if area.verify_containment(coordinate):
                # The point is inside the area
                found_data.append(('i', f'`{area.title}`.'))
            if area.verify_borderline(coordinate):
                # The point is on the border of the area
                found_data.append(('b', f'`{area.title}`.'))

        for line in self.lines:
            if coordinate == line.shape[0] or coordinate == line.shape[-1]:
                # The point is equal to an end point of the line
                found_data.append(('c', f'`{line.title}`.'))
            elif coordinate in line.shape:
                # The point is equal to a middle point of the line
                found_data.append(('c', f'`{line.title}`.'))
            else:
                for segment in line.segments:
                    if geometry.verify_collinearity(coordinate, segment):
                        # The point is collinear to a segment of the line
                        found_data.append(('c', f'`{line.title}`.'))

        return self._location_message(found_data, simplify)

    def _sort_areas(self) -> None:
        """
        Sorts `self.areas` by the top rightmost point of each area.
        """
        areas_u_r_m_points = {}
        for area in self.areas:
            area_all_points = []
            for point in area.polygon:
                if point not in area_all_points:
                    area_all_points.append(point)

            areas_u_r_m_points[
                self._upper_rightmost_point(area_all_points)
            ] = area

        areas_points_sorted = self._sort_points(areas_u_r_m_points.keys())
        sorted_areas = []
        for point in areas_points_sorted:
            sorted_areas.append(areas_u_r_m_points[point])

        self.areas = sorted_areas

    def _join_areas(self, areas=None) -> str | Area:
        """
        Joins the outer coordinates of the areas that share edges.
        :param areas: list of areas that may will be joined
        :return: A new `Area` object with the points that form the
        outer perimeter
        """
        if not areas:
            areas = self.areas

        all_unique_points, all_edges = (
            self._gather_unique_points_and_edges(areas)
        )
        # Asserting that none of the areas is separate from the rest
        if self._areas_are_separate(all_edges, areas):
            return MESSAGES[1]

        # Determining the upper-rightmost point
        u_r_m_point = self._upper_rightmost_point(all_unique_points)
        # Determining the count of unique edges
        ct_u_edges = self._count_unique_edges(all_edges)
        # Creating the outer polygon with unique edges and following
        # the sequence of edges
        outer_edges = self._create_outer_polygon(
            u_r_m_point, ct_u_edges, all_edges
        )
        # Creating the new polygon with the coordinates found
        new_polygon = [outer_edges[0][0]]
        for edge in outer_edges:
            new_polygon.append(edge[1])

        new_polygon.append(new_polygon[0])
        outer_area_title = MESSAGES[2].format(self.map_name)
        return Area(outer_area_title, new_polygon)

    def _gather_map(self, path: str) -> None:
        """
        Reads the `.geojson`/`.json` file, transforms it in json and  _
        gather the needed data, initiates an `Area` or `Line` for     _
        each shape found, append to its respective list.
        :param path: The location of the file.
        """
        self.lines = self._gather_lines(path)
        self.areas = self._gather_areas(path)

    def _sort_points(self, points: any) -> list:
        """
        Iterates through the points passed and orders from upper      _
        rightmost to bottom leftmost.
        :param points: <list(n)> of points to be sorted.
        :return: <list(n)> of sorted points.
        """
        sorted_points = []
        while True:
            highest_point = (-180, -90)
            for point in points:
                if (
                        point not in sorted_points and
                        self._point_is_higher(point, highest_point)
                ):
                    highest_point = point

            sorted_points.append(highest_point)

            if len(sorted_points) == len(points):
                break

        return sorted_points

    def _upper_rightmost_point(self, unique_points: list) -> tuple:
        """
        Iterates through the values in `unique_points`, verifying if  _
        `Map._point_is_higher()` returns True.
        :param unique_points: A <list(n)> of <tuples(2)> that contain _
        coordinates.
        :return: The upper-rightmost value in the list.
        """
        upper_rightmost_point = (-180, -90)
        for point in unique_points:
            if self._point_is_higher(point, upper_rightmost_point):
                upper_rightmost_point = point

        return upper_rightmost_point

    @staticmethod
    def _gather_unique_points_and_edges(list_of_areas: list) -> tuple:
        """
        Iterates through the list of areas collecting every pair of   _
        coordinates that are unique and the edges of every polygon    _
        from every area in `list_of_areas`.
        :param list_of_areas: <list(n)> of `Area` objects.
        :return: <tuple(2)> with the unique points [0] and the        _
        edges [1].
        """
        unique, borders = [], []
        for area in list_of_areas:
            for point in area.polygon:
                # Gathering every unique point
                if point not in unique:
                    unique.append(point)

            # Gathering every edge
            for border in area.borders:
                borders.append(border)

        return unique, borders

    @staticmethod
    def _location_message(results: list, simplify: bool) -> str:
        """
        Creates the output message for the findings of                _
        `Map.locate_point()`.
        :param results: The simplified version of the findings.
        :param simplify: The message will not be treated if True is   _
        passed.
        :return: A message of where the point was found.
        """
        if len(results) == 0:
            message = MESSAGES[3]
        elif simplify:
            message = ''
            for result in results:
                message += f'{result}\n'
        else:
            message = ''
            c = 1
            for result in results:
                phrase = ''
                if result[0] == 'i':
                    phrase = MESSAGES[5].format(result[1])
                elif result[0] == 'b':
                    phrase = MESSAGES[6].format(result[1])
                elif result[0] == 'c':
                    phrase = MESSAGES[7].format(result[1])
                message += f'\t{c}/{len(results)}:[{phrase}],'
                c += 1

            if c == 2:
                message = MESSAGES[4].format('this', '') + '{' + message + '}'
            else:
                message = MESSAGES[4].format('these', 's') + '{' + message + '}'

        return message

    @staticmethod
    def _areas_are_separate(all_edges: list, list_of_areas: list) -> bool:
        """
        Iterates through the areas in `list_of_areas` and counts its  _
        edges, verifying for any area that is separated from the rest.
        :param all_edges: All edges found in the group of areas.
        :param list_of_areas: All areas that it wishes to check for a _
        connected whole.
        :return: A boolean value for whether any of the areas is      _
        separated.
        """
        for area in list_of_areas:
            unique_count = 0
            borders_count = 0
            for border in area.borders:
                borders_count += 1
                if all_edges.count(border) > 1:
                    break  # At least one of the edges is not unique.
                else:
                    unique_count += 1  # The edge is unique

            if unique_count == borders_count:
                return True  # At least one area has the same count of_
                # edges and edges that are unique.

        return False  # All areas have at least one edge shared with  _
        # another

    @staticmethod
    def _point_is_higher(point: tuple, current_highest: tuple) -> bool:
        """
        Evaluates whether the candidate `point` is higher than the    _
        `current_higher` point, verifying if value for longitude of   _
        the former is greater than the latter or the longitudes are   _
        equal and the latitude of `point` is greater.
        :param point: the point that may be higher.
        :param current_highest: the highest point found so far.
        :return: Whether the point is upper rightmost from            _
        `current_higher`.
        """
        return (
            point[1] > current_highest[1] or (
                point[1] == current_highest[1] and
                point[0] > current_highest[0])
        )

    @staticmethod
    def _create_outer_polygon(
            up_rm_pt: tuple, ct_un_ed: int, edges: list
    ) -> list:
        """
        Iterates through `edges`, appending to a list the edge that   _
        is the next in the logical sequence. To be appended, the edge _
        must either be:
            - the first one to be added and its start the             _
            upper-rightmost point of the set; or
            - the edge must be unique in the set (considering that    _
            every inner-border's point is, at least, duplicated) and  _
            the endpoint of the last edge added is equal to the next  _
            one.
        Once the length of the new outer polygon reaches the count of _
        unique edges, the method halts, returning the finding.
        :param up_rm_pt: Upper-rightmost point in the set.
        :param ct_un_ed: Count of unique edges
        :param edges: All edges that are wished to form the outer     _
        polygon.
        :return: A <list(n)> of with the outer edges.
        """
        outer_edges = []
        last_edge = None
        while True:
            for edge in edges:
                if (
                        (not last_edge and edge[0] == up_rm_pt) or
                        (edges.count(edge) == 1 and last_edge[1] == edge[0])
                ):
                    outer_edges.append(edge)
                    last_edge = edge
                    break

            if len(outer_edges) == ct_un_ed:
                break

        return outer_edges

    @staticmethod
    def _count_unique_edges(edges: list) -> int:
        """
        Iterates through the values in `edges` adding to a count      _
        when the value isn't duplicated in the list.
        :param edges: <list(n)> of edges <tuple(2)<tuple(2)>>
        :return: The number of unique edges found.
        """
        count = 0
        for edge in edges:
            if edges.count(edge) == 1:
                count += 1

        return count

    @staticmethod
    def _gather_lines(path: str) -> list:
        """
        Reads the `.geojson`/`.json` file, transforms it in json and  _
        gather the needed data, initiates an `Line` for     _
        each shape found, append to the list.
        :param path: The location of the file
        :return: A list containing the `Line` objects found.
        """
        lines = []
        lines_dict = io.get_multiple_lines(path)
        for line_key in lines_dict:
            lines.append(Line(line_key, lines_dict[line_key]))

        return lines

    @staticmethod
    def _gather_areas(path: str) -> list:
        """
        Reads the `.geojson`/`.json` file, transforms it in json and  _
        gather the needed data, initiates an `Area` for     _
        each shape found, append to the list.
        :param path: The location of the file
        :return: A list containing the `Area` objects found.
        """
        areas = []
        areas_dict = io.get_multiple_areas(path)
        for area_key in areas_dict:
            areas.append(Area(area_key, areas_dict[area_key]))

        return areas



"""
    GeoBound
(PT-BR)
    Identificação de coordenadas e geolocalização de polígonos e linhas.
(EN-US)
    Identification of coordinates and geolocation of polygons and lines.

Copyright (C) 2024 Thiago Daniel Pessoa

This program is free software: you can redistribute it and/or modify it _
under the terms of the GNU General Public License as published by the   _
Free Software Foundation, either version 3 of the License, or (at your  _
option) any later version.

This program is distributed in the hope that it will be useful, but     _
WITHOUT ANY WARRANTY; without even the implied warranty of              _
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        _
General Public License for more details.

You should have received a copy of the GNU General Public License along _
with this program. If not, see <https://www.gnu.org/licenses/>.
"""


def check_coordinate(coordinate: tuple, _id=None) -> None:
    """
    Ensures that the coordinate set passed matches the criteria of a  _
    coordinate point.
    :param coordinate: <tuple(2)> of latitude and longitude
    :param _id: An identification for the point.
    """
    latitude, longitude = coordinate
    if not _id:
        _id = str((latitude, longitude))
    assert type(latitude) == type(longitude) == float, (
        f'Error in {_id}, the values for latitude and longitude must be float.'
    )
    assert -180 < latitude < 180, (
        f'Error in {_id}, the latitude (first value) of point {latitude} is '
        f'invalid (not between -180 and 180).'
    )
    assert -90 < longitude < 90, (
        f'Error in {_id}, the longitude (second value) of point {longitude} '
        f'is invalid (not between -90 and 90).'
    )


def digest_coordinates(title: str, coordinates: list) -> list:
    """
    Asserts that the values for latitude and longitude are valid
    and gets the coordinates of the polygon (often passed as
    `<list(3)>` [which are: lat, lng and alt; since altitude is
    irrelevant, for now, its discarded] to `<tuple(2)>`).
    :param title: The name of the structure
    :param coordinates: list(list(3))
    :return: list(tuple(2))
    """
    new_coordinates = []
    for coordinate in coordinates:
        check_coordinate(coordinate, title)
        new_point = (coordinate[0], coordinate[1])
        new_coordinates.append(new_point)

    return new_coordinates



"""
(PT-BR)
    Geolocalização e Georreferenciamento com poligonos
(EN-US)
    Geolocation and Georefferencing with polygons
Copyright (C) 2024 Thiago Daniel Pessoa

This program is free software: you can redistribute it and/or modify it _
under the terms of the GNU General Public License as published by the   _
Free Software Foundation, either version 3 of the License, or (at your  _
option) any later version.

This program is distributed in the hope that it will be useful, but     _
WITHOUT ANY WARRANTY; without even the implied warranty of              _
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        _
General Public License for more details.

You should have received a copy of the GNU General Public License along _
with this program. If not, see <https://www.gnu.org/licenses/>.
"""


import pathlib
from setuptools import (
    setup, 
    find_packages
)

setup(
    name='geo_bound',
    version='0.1.1',
    description='Location of coordinates in geospatial maps.',
    long_description=pathlib.Path('README.md').read_text(),
    long_description_content_type='text/markdown',
    url='https://github.com/TDPessoa/geolocation-with-coordinates',
    license='GPL-3.0 license',
    author='TDPessoa',
    author_email='thiago.d.pessoa@gmail.com',
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: Free To Use But Restricted',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
        'Topic :: Scientific/Engineering :: GIS',
        'Topic :: Scientific/Engineering :: Mathematics',
        'Topic :: Software Development :: Localization',
        'Topic :: Utilities'
    ],
    python_requires=">=3.11,<3.14",


)
